// *
// * $Id: CS2KPERFAdaptor.js 4163 2006-08-15 13:34:20Z wmartin $
// *
//  Licensed Materials - Property of IBM
//  5724-P69
// JSlater 13 Sept 2009 GWC SE Name parsing fixed.
// JSlater Sept 2009 - Formulas getValue fix applied
//  \251 Copyright IBM Corporation 2005, 2009. All Rights Reserved.
//  US Government Users Restricted Rights- Use, duplication or disclosure
//  restricted by GSA ADP Schedule Contract with IBM Corp.
// *


// ******************************************************
// * TRACES SETUP
// ******************************************************


PV.import("CS2KPERFConfig.js");

var DEBUG = true;

function logMessage(type, logdata) {
   if ( DEBUG ) {
       PV.Logger.logInfo("DEBUG", type, logdata);
   }
}



// ******************************************************
// * INIT GLOBAL VARIABLES DECLARATION
// ******************************************************
// *
// Because constructing Date is expensive (in any language),we'll only regenerate the Date
// object if the underlying strings have changed. A conversion function will remembers the
// last string values for the Date and Time fields and will remembers the last constructed
// ECMAScript Date object.
var laststringdate = "";
// var lastdateobject;

var currentSE;  // keep track of the current SE we're working on for the getValue function.

var recordCache = new Object();

var modelIntf = PV.ModelInterface();
PV.adaptor.modelInterface = modelIntf;
PV.adaptor.modelInterface.initializeForInventory();




function parseDate(date, time) {
    // 09-20-2004,10:00:08,BOSTON,OTS2,0,ORGTRMC,12,ORGTRMC2,0,ORGOUTC,2209,ORGOUTC2,0,INCTRMC,309,INCTRMC2,0
    // date = MM-DD-YYYY
    // time = HH:MM:SS
    // if ( ( laststringdate == ( date.toString() + time.toString() ) ) && ( laststringdate != "" ) ) {
        // logMessage("DATE_OBJECT", "Returning last date object 1", lastdateobject);
        // return lastdateobject;
    // } else if ( ! date || ! time) {
        // //       logMessage("DATE_OBJECT", "Returning last date object 2", lastdateobject);
        // return lastdateobject;
    // } else {
    var dateArr = date.split("-");
    var month = ( dateArr[0] - 1 );
    var day = dateArr[1];
    var year = dateArr[2];
    var timeArr = time.split(":");
    var hour = timeArr[0];
    var minute = timeArr[1];
    var second = timeArr[2];
    lastdateobject = new Date(year,month,day,hour,minute,0);
    logMessage("DATE_OBJECT", lastdateobject);
    return lastdateobject;
    // } 
}




function getLocation(record) {
   if ( locationHash[record.switchname] ) {
       return locationHash[record.switchname];
   } else {
       return "Unknown";
   }
}



// ******************************************************
// * CS2K PERF SECTION
// ******************************************************



function parseCS2KPERFFilename(source) {
  // File's name is something like:
  // customer_description.MM_DD_YYYY.HHMM.SWITCHNAME.GROUP.CSV

  var filename = source.uri.split("/").pop();
  var fields = filename.split(".");
  source.customerName = fields[0];
  var date = fields[1];
  var time = fields[2];
  source.nodeName = fields[3];
  var year = date.substring(6);
  var month = date.substring(0,2)-1;            //ECMAScript Jan = 0
  var day = date.substring(3,5);
  var hour = time.substring(0,2);
  var min = time.substring(2);
   var tstamp = new Date(year,month,day,hour,min);
  source.timestamp = tstamp;
 //  logMessage("DEBUG", "tstamp: " + tstamp);
  // source.timestamp = new Date(year,month,day,hour,min);
  source.localName = "NORTEL-" + filename;
  source.sortKey = source.timestamp.valueOf();
}


function handleCS2KPERF() {
    PV.templates.agg5.adapt(PV.event.value);
}

function getElementName(rec) {
  // return CS2KPERFInput.R.switchname;
  return rec.switchname;
}

function getSubelementName(switchname, info, type) {
   // Subelement name is supposed to be in PV.event.value.elementinfo from the record.
   // If the field's value is equal to 0, subelement is the device itself.

  // logMessage( "elementinfo=", info );

   var prefix = "CS2KPERF_";
   if ( info == "0" || type == "Device") {
      return prefix + switchname + '_Device';
   } 
   if ( type == "GWCA" ){
     // elementinfo is like GWC.0.UP
     var fields = info.split(".");
      var label = fields[1];
        logMessage("GWCA", label);
      return prefix + switchname + '_GWC_' + label ;
   } 
   if ( type == "GWCB" ){
     // elementinfo is like GWC 0
      var fields = info.split(" ");
      var label = fields[1];
        logMessage("GWCB", label);
      return prefix + switchname + '_GWC_' + label ;
   } else {
     // elementinfo is something like xxxx.label.2W.0.0
      var fields = info.split(".");
      var trunk = fields[0];
      var label = fields[1];
      return prefix + switchname + '_' + trunk + "_"+label + "_" + type;
   }
}


function getFamily() {
  return "VOIP_Softswitch";
}

function safeDivide( num1, num2 ) {
   if ( num2 == 0 ) {
      return 0;
   } else {
      return ( num1 / num2 );
   }
}


function getValue( rec, field ) {
      // recordCache[se].ts[rec.timestamp].fields[field] = numCheck;
      // check the current record, and then the recordCache
      logMessage("getValue", rec[field]);
      // if ( rec[field] == "" || rec[field] == undefined || rec[field] == null ) {
         // return undefined;
      // }
      var f = new Number(rec[field]);
      // if ( f == undefined || isNaN(f) || f < 0  || f == null ) {
      if ( rec[field] == "" || rec[field] == undefined || rec[field] == null ) {
          // check the recordCache...
          logMessage("getValue", "field does not contain a value.  Checking recordCache... field is: " + field);
          if ( recordCache[currentSE] ) {
             logMessage("recordCache", "recordCache has an entry for currentSE: " + currentSE);
             if ( recordCache[currentSE].ts[rec.timestamp].fields ) {
                logMessage("recordCache", "recordCache has an entry for currentSE: " + currentSE + " and 'fields' is : " + recordCache[currentSE].ts[rec.timestamp].fields);
                if ( recordCache[currentSE].ts[rec.timestamp].fields[field] > -1 && recordCache[currentSE].ts[rec.timestamp].fields[field] != null ) {
                    logMessage("recordCache", "returning value from recordCache for field: " + field + " value is: " + recordCache[currentSE].ts[rec.timestamp].fields[field]);
                    return recordCache[currentSE].ts[rec.timestamp].fields[field];
                }
             }
          }
          // don't create a 0 if there shouldn't be one.
          return undefined;
      } else {
          logMessage("FOUND DATA", "found data for field: " + field + " and value is: " + f);
          return f;
      }
}

// *
// *  device metrics hash
// *
// *   - defines formulas, subelements, and their associated metrics and properties
// *


var deviceMetrics = {
    Device: {
           properties: { VOIPSoftswitchComponentType: "Class 5 Softswitch", VOIPSoftswitchVendor: "Nortel",
                         VOIPSoftswitchLocation: function(rec) { return getLocation(rec) } },
           filter: function(rec) { return 1 },
           formulaPath: "AP~Specific~Bulk~Carrier VOIP~Infrastructure~Softswitch Platform~Class 5 Softswitch~Device",
           Formulas: {
                "CPUutilizationForCallProcessing%": { aggr: "average",
                     method: function(rec) { return ( getValue(rec, "BRSCAP")) } },
                "CPUutilizationAuxiliaryCallProcessing%": { aggr: "max",
                     method: function(rec) { return ( getValue(rec, "BRSAUXCP")) } },
                "No Circuit Trunk Available (Nb)": { aggr: "sum",
                     method: function(rec) { return ( getValue(rec, "OFZNCRT")) } },
                "IntraSwitchLineToLineCallAttempts": { aggr: "sum",
                     method: function(rec) { return ( getValue(rec, "ORGTRM")) } },
                 "IntraSwitchLineToLineSuccessfulCalls": { aggr: "sum",
                      method: function(rec) { return ( getValue(rec, "ORGTRMC")) } },
                 "IntraSwitchLineToLineSuccessfulCalls%": { aggr: "average",
                      method: function(rec) { return 100 * (  safeDivide(getValue(rec, "ORGTRMC"), getValue(rec, "ORGTRM") ) ) } },
                 "IntraSwitchLineToLineUnsuccessfulCalls": { aggr: "sum",
                      method: function(rec) { return ( getValue(rec, "ORGTRM") - getValue(rec, "ORGTRMC")) } },
                 "IntraSwitchLineToLineUnsuccessfulCalls%": { aggr: "average",
                      method: function(rec) { return 100 * ( safeDivide( getValue(rec, "ORGTRM") - getValue(rec, "ORGTRMC"), getValue(rec, "ORGTRM") ) ) } },
                "OutgoingLineToTrunkCallAttempts": { aggr: "sum",
                     method: function(rec) { return (getValue(rec, "ORGOUT")) } },
                 "OutgoingLineToTrunkSuccessfulCalls": { aggr: "sum",
                      method: function(rec) { return (getValue(rec, "ORGOUTC")) } },
                 "OutgoingLineToTrunkSuccessfulCalls%": { aggr: "average",
                      method: function(rec) { return 100 * ( safeDivide(getValue(rec, "ORGOUTC"), getValue(rec, "ORGOUT")) ) } },
                 "OutgoingLineToTrunkUnsuccessfulCalls": { aggr: "sum",
                      method: function(rec) { return ( getValue(rec, "ORGOUT") - getValue(rec, "ORGOUTC") ) } },
                 "OutgoingLineToTrunkUnsuccessfulCalls%": { aggr: "average",
                      method: function(rec) { return 100 * ( safeDivide( getValue(rec, "ORGOUT") - getValue(rec, "ORGOUTC"), getValue(rec, "ORGOUT")) ) } },
                "IncomingTrunkToLineCallAttempts": { aggr: "sum",
                     method: function(rec) { return (getValue(rec, "INCTRM")) } },
                 "IncomingTrunkToLineSuccessfulCalls": { aggr: "sum",
                      method: function(rec) { return (getValue(rec, "INCTRMC")) } },
                 "IncomingTrunkToLineSuccessfulCalls%": { aggr: "average",
                      method: function(rec) { return 100 * ( safeDivide(getValue(rec, "INCTRMC"), getValue(rec, "INCTRM") ) ) } },
                "IncomingTrunkToLineUnsuccessfulCalls": { aggr: "sum",
                     method: function(rec) { return ( getValue(rec, "INCTRM") - getValue(rec, "INCTRMC") ) } },
                 "IncomingTrunkToLineUnsuccessfulCalls%": { aggr: "average",
                      method: function(rec) { return 100 * ( safeDivide( getValue(rec, "INCTRM") - getValue(rec, "INCTRMC"), getValue(rec, "INCTRM") ) ) } },
                "TransitTrunkToTrunkCallAttempts": { aggr: "sum",
                     method: function(rec) { return (getValue(rec, "INCOUT")) } },
                 "TransitTrunkToTrunkSuccessfulCalls": { aggr: "sum",
                      method: function(rec) { return (getValue(rec, "INCOUTC")) } },
                 "TransitTrunkToTrunkSuccessfulCalls%": { aggr: "average",
                      method: function(rec) { return 100 * ( safeDivide(getValue(rec, "INCOUTC"), getValue(rec, "INCOUT")) ) } },
                 "TransitTrunkToTrunkUnsuccessfulCalls": { aggr: "sum",
                      method: function(rec) { return ( getValue(rec, "INCOUT") - getValue(rec, "INCOUTC") ) } },
                 "TransitTrunkToTrunkUnsuccessfulCalls%": { aggr: "average",
                      method: function(rec) { return 100 * ( safeDivide( getValue(rec, "INCOUT") - getValue(rec, "INCOUTC"), getValue(rec, "INCOUT") ) ) } },
                "TotalSoftswitchCallAttempts": { aggr: "sum",
                     method: function(rec) { return (getValue(rec, "ORGTRM") + getValue(rec, "ORGOUT") + getValue(rec, "INCTRM") + getValue(rec, "INCOUT") )} },
                "TotalSoftswitchSuccessfulCalls": { aggr: "sum",
                     method: function(rec) { return (getValue(rec, "ORGTRMC") + getValue(rec, "ORGOUTC") + getValue(rec, "INCTRMC") + getValue( rec, "INCOUTC")) } },
                "TotalSoftswitchSuccessfulCalls%": { aggr: "average",
                     method: function(rec) { return 100 * ( safeDivide(getValue(rec, "ORGTRMC") + getValue(rec, "ORGOUTC") + getValue(rec, "INCTRMC") + getValue(rec, "INCOUTC"), getValue(rec, "ORGTRM") + getValue(rec, "ORGOUT") + getValue(rec, "INCTRM") + getValue(rec, "INCOUT")) ) } },
                "TotalSoftswitchUnsuccessfulCalls": { aggr: "sum",
                     method: function(rec) { return ( getValue(rec, "ORGTRM") + getValue(rec, "ORGOUT") + getValue(rec, "INCTRM") + getValue(rec, "INCOUT") ) - ( getValue(rec, "ORGTRMC") + getValue(rec, "ORGOUTC") + getValue(rec, "INCTRMC") + getValue(rec, "INCOUTC") ) } },
                "TotalSoftswitchUnsuccessfulCalls%": { aggr: "average",
                     method: function(rec) { return 100 * ( safeDivide( getValue(rec, "ORGTRM") + getValue(rec, "ORGOUT") + getValue(rec, "INCTRM") + getValue(rec, "INCOUT") - getValue(rec, "ORGTRMC") + getValue(rec, "ORGOUTC") + getValue(rec, "INCTRMC") + getValue(rec, "INCOUTC"), getValue(rec, "ORGTRM") + getValue(rec, "ORGOUT") + getValue(rec, "INCTRM") + getValue(rec, "INCOUT" )) ) } },
                // 4.3W
                "CallAgentCPUUtilization": { aggr: "average",
                     method: function(rec) { return (getValue(rec, "BRSCAP")) } },
                "CallAgentMemoryUtilization": { aggr: "average",
                     method: function(rec) { return (getValue(rec, "SPAREMB")) } },
                "SparingBandwidth": { aggr: "average",
                     method: function(rec) { return (getValue(rec, "SP_MAX")) } },
                "ProcessorElement": { aggr: "average",
                     method: function(rec) { return (getValue(rec, "XASUTIL")) } },
                "HIOP": { aggr: "average",
                     method: function(rec) { return (getValue(rec, "IOUTIL")) } },
                "BusyCallHourAttempts": { aggr: "sum",
                     method: function(rec) { return ( getValue(rec, "NIN2") + ( getValue(rec, "NORIG2") * 65536 ) + getValue(rec, "NIN") + getValue(rec, "NORIG") ) } },
                "ENGCAP": { aggr: "average",
                     method: function(rec) {  var bcha = ( getValue(rec, "NIN2") + ( getValue(rec, "NORIG2") * 65536 ) + getValue(rec, "NIN") + getValue(rec, "NORIG") ); if ( getValue(rec, "XASUTIL") ) { return ( bcha / getValue(rec, "XASUTIL") ); } else { return ( bcha / getValue(rec, "BRSCAP") ) } } },
                "AUXCP": { aggr: "average",
                     method: function(rec) { return (getValue(rec, "XASAUXCP")) } },
                "SCAI": { aggr: "sum",
                     method: function(rec) { return ( ( getValue(rec, "SEIUMSGI") + getValue(rec, "SEIUMSGO") ) / 3600 ) } },
                "CallCompletionRatio_Tandem": { aggr: "average",
                     method: function(rec) { return ( 100 * ( safeDivide((  getValue(rec, "CONNECT2") * 65536 ) +  getValue(rec, "CONNECT"), ( getValue(rec, "NIN2") * 65536 ) + getValue(rec, "NIN" ))))}},
                "CallCompletionRatio_EndOffice": { aggr: "average",
                     method: function(rec) { return ( 1 - safeDivide(getValue(rec, "ORIGLKT") + getValue(rec, "INLKT") + getValue(rec, "ORIGBLK") + getValue(rec, "TERMBLK") + getValue(rec, "PERCLFL") + getValue(rec, "ORIGFAIL") +  getValue(rec, "INFAIL") + getValue(rec, "OUTFAIL") + getValue(rec, "CINTEGFL") + getValue(rec, "TRSGNCT") + getValue(rec, "TCMVACT") + getValue(rec, "TRSNCRT") + getValue(rec, "ORIGDENY") + getValue(rec, "CPLOOVFL") + getValue(rec, "CCBOVFL") + getValue(rec, "ORIGDENY"), getValue(rec, "NIN2") + ( getValue(rec, "NORIG2") * 65536 ) + getValue(rec, "NIN") + getValue(rec, "NORIG")) ) } },
                "NetworkEffectivenessRatio": { aggr: "average",
                     method: function(rec) { return ( 100 * ( (getValue( rec, "ANSWER2") * 65536 ) + ( safeDivide( getValue(rec, "ANSWERTRMTFR.TFRBUSY"), ( getValue(rec, "NIN2") * 65536  ) ) + getValue(rec, "NIN") ) ) ) } },
                "ServiceFailureRate": { aggr: "average",
                     method: function(rec) { return (  100 * safeDivide( getValue(rec, "INFAIL2") + getValue(rec, "OUTFAIL2") + ( getValue(rec, "NOVFLAT2") * 65536 ) + getValue(rec, "INFAIL")
 + getValue(rec, "OUTFAIL")  + ( getValue(rec, "NOVFLATB"), ( getValue(rec, "INCATOT2") + getValue(rec, "NATTEMP2") * 65536 ) + getValue(rec, "NATTMPT") + getValue(rec, "INCATOT") ) ) ) }
 },
                "AverageHoldingTime": { aggr: "average",
                     method: function(rec) { return (  getValue(rec, "LMTRU2") + ( getValue(rec, "TRU2") * 65536 ) + getValue( rec, "LMTRU") + ( safeDivide(( getValue(rec, "TRU") * 100 ), getValue(rec, "INCATOT2") )) + getValue(rec, "CONNECT2") + getValue(rec, "NORIGATT2") + ( getValue(rec, "NTERMATT2") * 65536 ) + getValue(rec, "INCATOT")  + getValue(rec, "CONNECT") + getValue(rec, "NTERMATT") +  getValue(rec, "NORIGATT")  - ( getValue(rec, "ORIGABN2") * 65536 ) - getValue(rec, "ORIGABN") ) } },
                "PacketLoss": { aggr: "average",
                      method: function(rec) { return ( 100* ( safeDivide(getValue(rec, "PKTLOSS"), ( getValue(rec, "INCATOT2") + getValue(rec, "CONNECT2") ) * 65536 + getValue(rec, "INCATOT")+ getValue(rec, "CONNECT" )) ) ) } },
                "Delay": { aggr: "average",
                     method: function(rec) { return ( 100 * ( safeDivide(getValue(rec, "DELAYQOS"), ( getValue(rec, "INCATOT2") + getValue(rec, "CONNECT2") ) * 65536 + getValue(rec, "INCATOT") + getValue(rec, "CONNECT") ) ) ) } },
                "Jitter": { aggr: "average",
                     method: function(rec) { return ( 100 * ( safeDivide(getValue(rec, "JITTER"), (getValue(rec, "INCATOT2") + getValue(rec, "CONNECT2")) * 65536 + getValue(rec, "INCATOT") + getValue(rec, "CONNECT") ) ) ) } },
                "LossOfEstablishedCalls": { aggr: "average",
                     method: function(rec) { return (getValue(rec, "CINTEGFL")) } },
                "TrunkOrCarrierBusy": { aggr: "sum",
                     method: function(rec) { return (getValue(rec, "TKPCBU")) } },
                "TrafficBusyUse": { aggr: "sum",
                     method: function(rec) { return (getValue(rec, "LMTRU")) } },
                "OriginatingLineCallAttempts": { aggr: "sum",
                     method: function(rec) { return (getValue(rec, "NORIGATT")) } },
                "OriginatingAbandonsBeforeConnection": { aggr: "sum",
                     method: function(rec) { return (getValue(rec, "ORIGABN")) } },
                "OriginatingFailures": { aggr: "sum",
                     method: function(rec) { return (getValue(rec, "ORIGBLK")) } },
                "OriginatingAttemptFailures": { aggr: "sum",
                     method: function(rec) { return (getValue(rec, "ORIGFAIL")) } },
                "TerminatingFailures": { aggr: "sum",
                     method: function(rec) { return (getValue(rec, "TERMBLK")) } }
           }
     },
     Trunk: {
           properties: { VOIPSoftswitchComponentType: "Class 5 Trunk", VOIPSoftswitchVendor: "Nortel",
                         VOIPSoftswitchLocation: function(rec) { return getLocation(rec) } },
           filter: function(rec) { if ( rec.groupname.indexOf("TRK") != -1 ) return 1 },
           formulaPath: "AP~Specific~Bulk~Carrier VOIP~Infrastructure~Softswitch Platform~Class 5 Softswitch~Trunk",
           Formulas: {
                "TrunkOverflow": { aggr: "sum", 
                    method: function(rec) { return (getValue(rec, "NOVFLATB")) } },
                "Incoming Call Bid Attempts (Nb)": { aggr: "sum", 
                    method: function(rec) { return (getValue(rec, "INCATOT")) } },
                "IncomingCallTrunkRoutingFailure": { aggr: "sum", 
                    method: function(rec) { return (getValue(rec, "INFAIL")) } },
                "IncomingCallTrunkRoutingFailure%": { aggr: "average", 
                    method: function(rec) { return 100 * ( safeDivide(getValue(rec, "INFAIL"), getValue(rec, "INCATOT")) ) } },
                "IncomingSuccessfulCalls": { aggr: "sum", 
                    method: function(rec) { return (getValue(rec, "INANSWER")) } },
                "IncomingSuccessfulCalls%": { aggr: "average", 
                    method: function(rec) { logMessage("incoming successful calls %", "inanswer: " + getValue(rec, "INANSWER") + " incatot: " + getValue(rec, "INCATOT") ); return 100 * ( safeDivide(getValue(rec, "INANSWER"), getValue(rec, "INCATOT")) ) } },
                "IncomingAnswerSeizureRatio%": { aggr: "average", 
                    method: function(rec) { return 100 * ( safeDivide(getValue(rec, "ANSWER") + getValue(rec, "INANSWER"),  getValue(rec, "CONNECT") + getValue(rec, "INCATOT") ) ) } },
                "Outgoing Call Bid Attempts (Nb)": { aggr: "sum", 
                    method: function(rec) { return ( getValue(rec, "NATTMPT")) } },
                "OutgoingCallTrunkRoutingFailure": { aggr: "sum", 
                    method: function(rec) { return ( getValue(rec, "NORGOUT") - getValue(rec, "ANSWER") ) } },
                // "OutgoingCallTrunkRoutingFailure%": { aggr: "average", 
                  // method: function(rec) { return 100 * ( safeDivide(getValue(rec, "NORGOUT") - getValue(rec, "ANSWER"),  getValue(rec, "NORGOUT")) ) } },
                "OutgoingSuccessfulCalls": { aggr: "sum", 
                    method: function(rec) { return ( getValue(rec, "NATTMPT") - getValue(rec, "OUTFAIL") ) } },
                "OutgoingSuccessfulCalls%": { aggr: "average", 
                    method: function(rec) { return 100 * ( safeDivide(getValue(rec, "NATTMPT") - getValue(rec, "OUTFAIL"), getValue(rec, "NATTMPT") ) ) } },
                "OutgoingAnswerSeizureRatio%": { aggr: "average", 
                    method: function(rec) { return ( 100 * ( safeDivide(getValue( rec, "NATTMPT") - getValue(rec, "OUTFAIL"),  getValue(rec, "ANSWER") ) ) ) } },
                "TotalCallBidAttempts": { aggr: "sum", 
                    method: function(rec) { return ( getValue(rec, "INCATOT") + getValue(rec, "NATTMPT") ) } },
                "TotalCallTrunkRoutingFailure": { aggr: "sum", 
                    method: function(rec) { return ( getValue(rec, "INCATOT") - getValue(rec, "NINCTRM") ) + ( getValue(rec, "NORGOUT")- getValue(rec, "ANSWER") ) } },
                "TotalCallTrunkRoutingFailure%": { aggr: "average", 
                    method: function(rec) { return 100 * ( safeDivide( getValue(rec, "INFAIL") + getValue(rec, "OUTFAIL"), getValue(rec, "NATTMPT") ) ) } },
                // "TotalSuccesfulCalls": { aggr: "sum", 
                    // method: function(rec) { return rec.CARDUTILAVG } },
                // "TotalSuccesfulCalls%": { aggr: "average", 
                    // method: function(rec) { return rec.CARDUTILAVG } },
                // "TotalAnswerSeizureRatio%": { aggr: "average", 
                    // method: function(rec) { return rec.CARDUTILAVG } }
                 // 4.3W formulas
                "TrunkGOS": { aggr: "average",
                      method: function(rec) { return ( safeDivide( ( getValue(rec, "TRU2") * 65536 ) + getValue(rec, "TRU"), getValue(rec, "NWCCT")) ) } }
           }
     },
     GWCA: {
             properties: { VOIPSoftswitchComponentType: "Class 5 GWC", VOIPSoftswitchVendor: "Nortel",
                         VOIPSoftswitchLocation: function(rec) { return getLocation(rec) } },
           filter: function(rec) { if ( rec.groupname.indexOf("XPMOCC") != -1 || rec.groupname.indexOf("XPMOCC2") != -1 ) return 1 },
           formulaPath: "AP~Specific~Bulk~Carrier VOIP~Infrastructure~Softswitch Platform~Class 5 Softswitch~GWC",
           Formulas: {
                "BHHCA": { aggr: "sum",
                      method: function(rec) { return ( getValue(rec, "PMORIGS") + getValue(rec, "PMTERMS") + getValue(rec, "PMORIGS1") + getValue( rec, "PMORIGS2") + getValue(rec, "PMTERMS1") + getValue(rec, "PMTERMS2") ) } },
                "Processor": { aggr: "sum",
                      method: function(rec) { return ( safeDivide(getValue(rec, "AVGCPOCC"), getValue(rec, "NUMRPTS") ) ) } }
        //        "CCR": { aggr: "sum",
        //              method: function(rec) { return ( 100 * ( ( rec.PKTNTWCF + rec.ITACFL + rec.VCACCFL + rec.DQOSCFL + rec.OTHERCFL ) / ( ( rec.PACKETC2 * 65536 ) + rec.PACKETCA ) ) ) } },
        //        "Mid-call Failure Rate": { aggr: "sum",
        //              method: function(rec) { return ( 100 * ( ( rec.PKTDENDO + rec.PKTDENDT ) / ( ( rec.PACKETC2 * 65536 ) + rec.PACKETCA ) ) ) } }
           }
    },
GWCB: {
             properties: { VOIPSoftswitchComponentType: "Class 5 GWC", VOIPSoftswitchVendor: "Nortel",
                         VOIPSoftswitchLocation: function(rec) { return getLocation(rec) } },
           filter: function(rec) { if ( rec.groupname.indexOf("XPKTFLOM") != -1  )
 return 1 },
           formulaPath: "AP~Specific~Bulk~Carrier VOIP~Infrastructure~Softswitch Platform~Class 5 Softswitch~GWC",
           Formulas: {
   //             "BHHCA": { aggr: "sum",
   //                   method: function(rec) { return ( rec.PMORIGS + rec.PMTERMS + rec.PMORIGS1 + rec.PMORIGS2 + rec.PMTERMS1 + rec.PMTERMS2 ) } },
   //             "Processor": { aggr: "sum",
   //                   method: function(rec) { return safeDivide( rec.AVGCPOCC, rec.NUMRPTS ) } },
                "CCR": { aggr: "sum",
                      method: function(rec) { return 100 * safeDivide( getValue(rec, "PKTNTWCF") + getValue(rec, "ITACFL") + getValue(rec, "VCACCFL") + getValue(rec, "DQOSCFL") + getValue(rec, "OTHERCFL"), ( getValue(rec, "PACKETC2") * 65536 ) + getValue(rec, "PACKETCA") ) } },
                "Mid-call Failure Rate": { aggr: "sum",
                      method: function(rec) { return ( 100 * ( safeDivide( getValue(rec, "PKTDENDO") + getValue(rec, "PKTDENDT"), ( getValue(rec, "PACKETC2") * 65536 ) + getValue(rec, "PACKETCA" )) ) ) } }
           }
    },

    MS2010: {
             properties: { VOIPSoftswitchComponentType: "Class 5 MS2010", VOIPSoftswitchVendor: "Nortel",
                         VOIPSoftswitchLocation: function(rec) { return getLocation(rec) } },
           filter: function(rec) { if ( rec.groupname.indexOf("AUDSRVS") != -1 || rec.groupname.indexOf("ANN") != -1 ||rec.groupname.indexOf("CF3P") != -1 || rec.groupname.indexOf("CF6P") != -1 )return 1 },
           formulaPath: "AP~Specific~Bulk~Carrier VOIP~Infrastructure~Softswitch Platform~Class 5 Softswitch~MS2010",
           Formulas: {
                "ANNocc": { aggr: "sum",
                      method: function(rec) { return ( 100 * safeDivide( getValue(rec, "ANNCTRU" ), getValue(rec, "ANNCINSU") ) ) } },
                "3PortOcc": { aggr: "sum",
                      method: function(rec) { return ( 100 * safeDivide( getValue(rec, "CNF3TRU"), getValue(rec, "CNF3INSU") ) ) } },
                "6PortOcc": { aggr: "sum",
                      //  there doesn't appear to be a CF6INSU in the docs. pending simon information this will be changed
                      method: function(rec) { return ( 100 * safeDivide( getValue(rec, "CNF6TRU"), getValue(rec, "CNF6INSU") ) ) } },
                "ANNFailureRatio": { aggr: "sum",
                      method: function(rec) { return ( 100 * safeDivide(getValue( rec, "ANNOVFL"), getValue(rec, "ANNATT") ) ) } },
                "3PortConfFailureRatio": { aggr: "sum",
                      method: function(rec) { return ( 100 * safeDivide( getValue(rec, "CNFOVFL"), getValue( rec, "CNFSZRS") ) ) } },
                "6PortConfFailureRatio": { aggr: "sum",
                      method: function(rec) { return ( 100 * safeDivide( getValue(rec, "CF6OVFL"), getValue(rec, "CF6SZRS") ) ) } }

           }
    }


};



// *
// *  These records are of varying length and content, so we must process them to see what they
// *  are, and stuff back named data in the record hash
// *
// *


// 09-20-2004,10:00:08,BOSTON,OTS2,0,ORGTRMC,12,ORGTRMC2,0,ORGOUTC,2209,ORGOUTC2,0,INCTRMC,309,INCTRMC2,0
// 09-20-2004,10:00:08,BOSTON,OTS,0,ORGTRM,17,ORGTRM2,0,ORGOUT,2864,ORGOUT2,0,INCTRM,429,INCTRM2,0,INCOUT,2129
// 09-20-2004,10:00:08,BOSTON,TRK2,52.LOWELLNET.2W.0.0,NORGOUT,0,NORGOUTC,0,NINCTRM,0,NINCTRMC,0
// 09-20-2004,10:00:08,BOSTON,TRK2,53.SFOINET.2W.0.0,NORGOUT,0,NORGOUTC,0,NINCTRM,0,NINCTRMC,0
// 09-20-2004,10:00:08,BOSTON,TRK2,54.SFOINET.2W.0.0,NORGOUT,0,NORGOUTC,0,NINCTRM,0,NINCTRMC,0
// 09-20-2004,10:00:08,BOSTON,TRK2,55.TEST.2W.0.0,NORGOUT,0,NORGOUTC,0,NINCTRM,0,NINCTRMC,0
// 09-20-2004,10:00:08,BOSTON,TRK2,56.OA_1.2W.2.0,NORGOUT,0,NORGOUTC,0,NINCTRM,0,NINCTRMC,0
// 09-20-2004,10:00:08,BOSTON,TRK2,116.SFO.2W.0.0,NORGOUT,0,NORGOUTC,0,NINCTRM,0,NINCTRMC,0
// 09-20-2004,10:00:08,BOSTON,TRK2,420.NYC200_1.2W.6.2,NORGOUT,0,NORGOUTC,0,NINCTRM,0,NINCTRMC,0
// 09-20-2004,10:00:08,BOSTON,TRK2,423.NYC400_1.2W.20.20,NORGOUT,0,NORGOUTC,0,NINCTRM,0,NINCTRMC,0
// 09-20-2004,10:00:08,BOSTON,TRK2,998.LOWEL01M_FOO.2W.31.31,NORGOUT,111,NORGOUTC,89,NINCTRM,0,NINCTRMC,0


function cacheFieldsForSubelement(rec, se) {
   if ( recordCache[se] == null || recordCache[se] == undefined ) {
      recordCache[se] = new Object();
      recordCache[se].ts = new Object();
      recordCache[se].ts[rec.timestamp] = new Object();
      recordCache[se].ts[rec.timestamp].fields = new Object();
   }
   if ( recordCache[se].ts[rec.timestamp] == undefined ) {
      recordCache[se].ts[rec.timestamp] = new Object();
      recordCache[se].ts[rec.timestamp].fields = new Object();
   }
   logMessage("[cacheFieldsForSubelement]", rec);
   // apply the fields and their values to the recordCache[se].fields object.
   for ( var field in rec ) {
       // check to see if there's a number value.
       var numCheck = new Number(rec[field]);
       if ( ! isNaN(numCheck) ) {
           // this is probably an OM to cache
           if ( recordCache[se].ts[rec.timestamp].fields == undefined || ! recordCache[se].ts[rec.timestamp].fields ) {
                recordCache[se].ts[rec.timestamp].fields = new Object();
           }
           logMessage("recordCache", "cacheing field: " + field + " with value: " + numCheck + " for se: " + se);
           recordCache[se].ts[rec.timestamp].fields[field] = numCheck;
       }
   }
}


function processRecord(rec) {
   var metric;
   var subelementName;
   var subelement;

   var eName = getElementName(rec);
   if ( ! eName ) {
   //   logMessage("IGNORING_EMPTY_RECORD", "RECORD_IGNORE");
      return;
   }

   var element = modelIntf.Element();
   element.state = true;
   element.name = eName;

   var ts = parseDate(rec.date, rec.time);
   rec.timestamp = ts;
   // logMessage("ELEMENT_NAME", eName);
   element.origin = "CS2KPERF";
   // TODO - use a generic timestamp for the IPDR side of the data files
   // element.timestamp = parseDate(rec.date, rec.time);
   element.timestamp = ts;
   // store persistent data, and stuff information back into the record object that we may
   // need later.
   arrangeRecordData(rec);

   for ( var deviceType in deviceMetrics ) {
        //     logMessage("DEVICE_TYPE", deviceType);
        if ( deviceMetrics[deviceType].filter(rec) ) {
           subelementName = getSubelementName(eName, rec.elementinfo, deviceType);
           logMessage("SUBELEMENT_NAME", subelementName);
           subelement = modelIntf.Subelement();
           subelement.name = subelementName;
           currentSE = subelementName;
           cacheFieldsForSubelement(rec, subelementName);
           subelement.invariant = subelementName;
           subelement.instance = subelementName;
           subelement.family = getFamily(rec);
           subelement.label = subelementName;
           subelement.timestamp = ts;
           subelement.origin = "CS2KCDR";
           //       logMessage("ADDING_SUBELEMENT", "1");
           for ( var property in deviceMetrics[deviceType].properties ) {
               if ( typeof deviceMetrics[deviceType].properties[property] == "function" ) {
                    //               logMessage("ADDING_PROPERTY_FUNCTION", property.toString());
                    subelement.addProperty(property.toString(), deviceMetrics[deviceType].properties[property](rec));
               } else {
                    //              logMessage("ADDING_PROPERTY_NON_FUNCTION", property.toString());
                    subelement.addProperty(property.toString(), deviceMetrics[deviceType].properties[property]);
               }
           }
           element.addSubelement(subelement);
           //    logMessage("SUBELEMENT_ADDED", "1");
           for ( var form in deviceMetrics[deviceType].Formulas ) {
               //       logMessage("METRIC_PROCESSING", form);
               //      logMessage("FORMULA_PATH_HERE", deviceMetrics[deviceType].formulaPath + "~" + form);
               logMessage("METRIC_DATE", ts);
               logMessage("METRIC_NAME", form);
               //     logMessage("FORMULA_METHOD", "1");
               // var val = toval(deviceMetrics[deviceType].Formulas[form].method(rec));
               var val = deviceMetrics[deviceType].Formulas[form].method(rec);
               val = new Number(val);
               if ( isFinite(val) ) {
                  metric = modelIntf.FloatMetric();
                  metric.timestamp = ts;
                  metric.name = deviceMetrics[deviceType].formulaPath + "~" + form;
                  // metric.aggregationInterval = 900;
                  // metric.aggregation = deviceMetrics[deviceType].Formulas[form].aggr;
                  metric.value = val;
                  //       logMessage("FORMULA_METHOD", deviceMetrics[deviceType].Formulas[form].method(rec));
                  // subelement.addAggregatingMetric(metric);
                  subelement.addMetric(metric);
                  //      logMessage("FORMULA_METHOD", "2");
               } else {
                  logMessage("FOUND_INVALID", "metric value invalid: " + val);
               }
           }
           // logMessage("OUTSIDE_OF_FORMULA_LOOP", "!!!");
        }
    }
    modelIntf.commit();
}

//method added out of IFIX
function toval(regval) {
   var num = (regval-0);
   if (!isNaN(num)) {
       return num;
   }
   return 0;
}

// ------------------------------------------
// ARRANGE THE DYNAMIC CONTENTS OF THE RECORD
// ------------------------------------------

function arrangeRecordData(rec) {
// [IN_ARRANGE_RECORDS]    DEBUG   Record(date:'11-02-2009', elementinfo:'0', groupname:'OTS2', RegName_1:'NANSATT', RegName_10:'ORGTRMC2', RegName_11:'ORGOUTC', RegName_12:'ORGOUTC2', RegName_13:'INCTRMC', RegName_14:'INCTRMC2', RegName_15:'INCOUTC', RegName_16:'INCOUTC2', RegName_17:'NALRTE', RegName_18:'AVTRKHLD', RegName_19:'AVREGHLD', RegName_2:'NANSATT2', RegName_20:'ANATTMP', RegName_21:'ACONNEC', RegName_22:'NBLKC', RegName_23:'ORGTRMU', RegName_24:'ORGTRMU2', RegName_3:'NUNAATT', RegName_4:'NUNAATT2', RegName_5:'NBSYATT', RegName_6:'NBSYATT2', RegName_7:'NOUTSZ', RegName_8:'NOUTSZ2', RegName_9:'ORGTRMC', RegValue_1:0, RegValue_10:0, RegValue_11:0, RegValue_12:0, RegValue_13:0, RegValue_14:0, RegValue_15:0, RegValue_16:0, RegValue_17:0, RegValue_18:0, RegValue_19:0, RegValue_2:0, RegValue_20:0, RegValue_21:0, RegValue_22:0, RegValue_23:0, RegValue_24:0, RegValue_3:0, RegValue_4:0, RegValue_5:0, RegValue_6:0, RegValue_7:0, RegValue_8:0, RegValue_9:0, switchname:'BNR_LAB_LOAD', time:'14:00:02')

     logMessage("IN_ARRANGE_RECORDS", rec);
     var attrName;
     var tmpHash = {};
     for ( var data in rec ) {
         logMessage("VAR_DATA_IN_REC", data);
         // Register values are after this point..
         if ( data.indexOf("Reg") != -1 ) {
             var tmpArr = data.split("_");
             var num = tmpArr[1];
             if ( ! tmpHash[num] ) {
                tmpHash[num] = new Object();
             }
             if ( data.indexOf("Name") != -1 ) {
                logMessage("data is RegName", rec[data]);
                tmpHash[num].name = rec[data];
             } else {
                logMessage("data is RegValue", ( rec[data] - 0 ));
                tmpHash[num].value = new Number(rec[data]-0);
                if ( isNaN(tmpHash[num].value) ) {
                   // this 'waste' message is a waste.  the name may not have been registered yet.  
                   //  since the record object is a hash, we won't necessarily see the names before the
                   //   values 
                   logMessage("WASTE",tmpHash[num].name);
                   // tmpHash[num].value = 0;
                }
             }
             delete rec[data];
             if ( tmpHash[num].name && ( tmpHash[num].value > -1 ) && ! isNaN(tmpHash[num].value) ) {
                  logMessage("COMPLETED RECORD FOR: ", tmpHash[num].name + " " + tmpHash[num].value);
                  rec[tmpHash[num].name] = tmpHash[num].value;
                  delete tmpHash[num];
             }
         }
     }
    logMessage("RECORD_IS_NOW_______", rec);
}



// **********
// * Schema *
// **********


PV.adaptor.initializeSchemas = function initializeSchemas() {
   this.schemas.add(
     PV.TextInputSchema("cs2kperf", { onOpen: function() { PV.adaptor.modelInterface = modelIntf; recordCache = new Object(); },
                                      onClose: function() { modelIntf.commit() }, parseURI: function(source) { parseCS2KPERFFilename(source) } },
        PV.DelimitedRecordSchema("recordSchema", { fieldSeparator: ",", recordSeparator: "\r", maxOccurs: Infinity, hasHeaderRecord: false,
                                                   onRead: function(rec) { processRecord(rec) } },
            PV.StringFieldSchema("date"),
            PV.StringFieldSchema("time"),
            PV.StringFieldSchema("switchname"),
            PV.StringFieldSchema("groupname"),
            PV.StringFieldSchema("elementinfo"),
            PV.StringFieldSchema("RegName_1"),
            PV.NumberFieldSchema("RegValue_1"),
            PV.StringFieldSchema("RegName_2"),
            PV.NumberFieldSchema("RegValue_2"),
            PV.StringFieldSchema("RegName_3"),
            PV.NumberFieldSchema("RegValue_3"),
            PV.StringFieldSchema("RegName_4"),
            PV.NumberFieldSchema("RegValue_4"),
            PV.StringFieldSchema("RegName_5"),
            PV.NumberFieldSchema("RegValue_5"),
            PV.StringFieldSchema("RegName_6"),
            PV.NumberFieldSchema("RegValue_6"),
            PV.StringFieldSchema("RegName_7"),
            PV.NumberFieldSchema("RegValue_7"),
            PV.StringFieldSchema("RegName_8"),
            PV.NumberFieldSchema("RegValue_8"),
            PV.StringFieldSchema("RegName_9"),
            PV.NumberFieldSchema("RegValue_9"),
            PV.StringFieldSchema("RegName_10"),
            PV.NumberFieldSchema("RegValue_10"),
            PV.StringFieldSchema("RegName_11"),
            PV.NumberFieldSchema("RegValue_11"),
            PV.StringFieldSchema("RegName_12"),
            PV.NumberFieldSchema("RegValue_12"),
            PV.StringFieldSchema("RegName_13"),
            PV.NumberFieldSchema("RegValue_13"),
            PV.StringFieldSchema("RegName_14"),
            PV.NumberFieldSchema("RegValue_14"),
            PV.StringFieldSchema("RegName_15"),
            PV.NumberFieldSchema("RegValue_15"),
            PV.StringFieldSchema("RegName_16"),
            PV.NumberFieldSchema("RegValue_16"),
            PV.StringFieldSchema("RegName_17"),
            PV.NumberFieldSchema("RegValue_17"),
            PV.StringFieldSchema("RegName_18"),
            PV.NumberFieldSchema("RegValue_18"),
            PV.StringFieldSchema("RegName_19"),
            PV.NumberFieldSchema("RegValue_19"),
            PV.StringFieldSchema("RegName_20"),
            PV.NumberFieldSchema("RegValue_20"),
            PV.StringFieldSchema("RegName_21"),
            PV.NumberFieldSchema("RegValue_21"),
            PV.StringFieldSchema("RegName_22"),
            PV.NumberFieldSchema("RegValue_22"),
            PV.StringFieldSchema("RegName_23"),
            PV.NumberFieldSchema("RegValue_23"),
            PV.StringFieldSchema("RegName_24"),
            PV.NumberFieldSchema("RegValue_24"),
            PV.StringFieldSchema("RegName_25"),
            PV.NumberFieldSchema("RegValue_25"),
            PV.StringFieldSchema("RegName_26"),
            PV.NumberFieldSchema("RegValue_26"),
            PV.StringFieldSchema("RegName_27"),
            PV.NumberFieldSchema("RegValue_27"),
            PV.StringFieldSchema("RegName_28"),
            PV.NumberFieldSchema("RegValue_28"),
            PV.StringFieldSchema("RegName_29"),
            PV.NumberFieldSchema("RegValue_29"),
            PV.StringFieldSchema("RegName_30"),
            PV.NumberFieldSchema("RegValue_30"),
            PV.StringFieldSchema("RegName_31"),
            PV.NumberFieldSchema("RegValue_31"),
            PV.StringFieldSchema("RegName_32"),
            PV.NumberFieldSchema("RegValue_32"),
            PV.StringFieldSchema("RegName_33"),
            PV.NumberFieldSchema("RegValue_33"),
            PV.StringFieldSchema("RegName_34"),
            PV.NumberFieldSchema("RegValue_34"),
            PV.StringFieldSchema("RegName_35"),
            PV.NumberFieldSchema("RegValue_35"),
            PV.StringFieldSchema("RegName_36"),
            PV.NumberFieldSchema("RegValue_36"),
            PV.StringFieldSchema("RegName_37"),
            PV.NumberFieldSchema("RegValue_37"),
            PV.StringFieldSchema("RegName_38"),
            PV.NumberFieldSchema("RegValue_38"),
            PV.StringFieldSchema("RegName_39"),
            PV.NumberFieldSchema("RegValue_39"),
            PV.StringFieldSchema("RegName_40"),
            PV.NumberFieldSchema("RegValue_40"),
            PV.StringFieldSchema("RegName_41"),
            PV.NumberFieldSchema("RegValue_41"),
            PV.StringFieldSchema("RegName_42"),
            PV.NumberFieldSchema("RegValue_42"),
            PV.StringFieldSchema("RegName_43"),
            PV.NumberFieldSchema("RegValue_43"),
            PV.StringFieldSchema("RegName_44"),
            PV.NumberFieldSchema("RegValue_44"),
            PV.StringFieldSchema("RegName_45"),
            PV.NumberFieldSchema("RegValue_45"),
            PV.StringFieldSchema("RegName_46"),
            PV.NumberFieldSchema("RegValue_46"),
            PV.StringFieldSchema("RegName_47"),
            PV.NumberFieldSchema("RegValue_47"),
            PV.StringFieldSchema("RegName_48"),
            PV.NumberFieldSchema("RegValue_48"),
            PV.StringFieldSchema("RegName_49"),
            PV.NumberFieldSchema("RegValue_49"),
            PV.StringFieldSchema("RegName_50"),
            PV.NumberFieldSchema("RegValue_50"),
            PV.StringFieldSchema("RegName_51"),
            PV.NumberFieldSchema("RegValue_51"),
            PV.StringFieldSchema("RegName_52"),
            PV.NumberFieldSchema("RegValue_52"),
            PV.StringFieldSchema("RegName_53"),
            PV.NumberFieldSchema("RegValue_53"),
            PV.StringFieldSchema("RegName_54"),
            PV.NumberFieldSchema("RegValue_54"),
            PV.StringFieldSchema("RegName_55"),
            PV.NumberFieldSchema("RegValue_55"),
            PV.StringFieldSchema("RegName_56"),
            PV.NumberFieldSchema("RegValue_56"),
            PV.StringFieldSchema("RegName_57"),
            PV.NumberFieldSchema("RegValue_57"),
            PV.StringFieldSchema("RegName_58"),
            PV.NumberFieldSchema("RegValue_58"),
            PV.StringFieldSchema("RegName_59"),
            PV.NumberFieldSchema("RegValue_59"),
            PV.StringFieldSchema("RegName_60"),
            PV.NumberFieldSchema("RegValue_60"),
            PV.StringFieldSchema("RegName_61"),
            PV.NumberFieldSchema("RegValue_61"),
            PV.StringFieldSchema("RegName_62"),
            PV.NumberFieldSchema("RegValue_62"),
            PV.StringFieldSchema("RegName_63"),
            PV.NumberFieldSchema("RegValue_63"),
            PV.StringFieldSchema("RegName_64"),
            PV.NumberFieldSchema("RegValue_64"),
            PV.StringFieldSchema("RegName_65"),
            PV.NumberFieldSchema("RegValue_65"),
            PV.StringFieldSchema("RegName_66"),
            PV.NumberFieldSchema("RegValue_66"),
            PV.StringFieldSchema("RegName_67"),
            PV.NumberFieldSchema("RegValue_67"),
            PV.StringFieldSchema("RegName_68"),
            PV.NumberFieldSchema("RegValue_68"),
            PV.StringFieldSchema("RegName_69"),
            PV.NumberFieldSchema("RegValue_69"),
            PV.StringFieldSchema("RegName_70"),
            PV.NumberFieldSchema("RegValue_70"),
            PV.StringFieldSchema("RegName_71"),
            PV.NumberFieldSchema("RegValue_71"),
            PV.StringFieldSchema("RegName_72"),
            PV.NumberFieldSchema("RegValue_72"),
            PV.StringFieldSchema("RegName_73"),
            PV.NumberFieldSchema("RegValue_73"),
            PV.StringFieldSchema("RegName_74"),
            PV.NumberFieldSchema("RegValue_74"),
            PV.StringFieldSchema("RegName_75"),
            PV.NumberFieldSchema("RegValue_75"),
            PV.StringFieldSchema("RegName_76"),
            PV.NumberFieldSchema("RegValue_76"),
            PV.StringFieldSchema("RegName_77"),
            PV.NumberFieldSchema("RegValue_77"),
            PV.StringFieldSchema("RegName_78"),
            PV.NumberFieldSchema("RegValue_78"),
            PV.StringFieldSchema("RegName_79"),
            PV.NumberFieldSchema("RegValue_79"),
            PV.StringFieldSchema("RegName_80"),
            PV.NumberFieldSchema("RegValue_80"),
            PV.StringFieldSchema("RegName_81"),
            PV.NumberFieldSchema("RegValue_81"),
            PV.StringFieldSchema("RegName_82"),
            PV.NumberFieldSchema("RegValue_82"),
            PV.StringFieldSchema("RegName_83"),
            PV.NumberFieldSchema("RegValue_83"),
            PV.StringFieldSchema("RegName_84"),
            PV.NumberFieldSchema("RegValue_84"),
            PV.StringFieldSchema("RegName_85"),
            PV.NumberFieldSchema("RegValue_85"),
            PV.StringFieldSchema("RegName_86"),
            PV.NumberFieldSchema("RegValue_86"),
            PV.StringFieldSchema("RegName_87"),
            PV.NumberFieldSchema("RegValue_87"),
            PV.StringFieldSchema("RegName_88"),
            PV.NumberFieldSchema("RegValue_88"),
            PV.StringFieldSchema("RegName_89"),
            PV.NumberFieldSchema("RegValue_89"),
            PV.StringFieldSchema("RegName_90"),
            PV.NumberFieldSchema("RegValue_90"),
            PV.StringFieldSchema("RegName_91"),
            PV.NumberFieldSchema("RegValue_91"),
            PV.StringFieldSchema("RegName_92"),
            PV.NumberFieldSchema("RegValue_92"),
            PV.StringFieldSchema("RegName_93"),
            PV.NumberFieldSchema("RegValue_93"),
            PV.StringFieldSchema("RegName_94"),
            PV.NumberFieldSchema("RegValue_94"),
            PV.StringFieldSchema("RegName_95"),
            PV.NumberFieldSchema("RegValue_95"),
            PV.StringFieldSchema("RegName_96"),
            PV.NumberFieldSchema("RegValue_96"),
            PV.StringFieldSchema("RegName_97"),
            PV.NumberFieldSchema("RegValue_97"),
            PV.StringFieldSchema("RegName_98"),
            PV.NumberFieldSchema("RegValue_98"),
            PV.StringFieldSchema("RegName_99"),
            PV.NumberFieldSchema("RegValue_99"),
            PV.StringFieldSchema("RegName_100"),
            PV.NumberFieldSchema("RegValue_100"),
            PV.StringFieldSchema("RegName_101"),
            PV.NumberFieldSchema("RegValue_101"),
            PV.StringFieldSchema("RegName_102"),
            PV.NumberFieldSchema("RegValue_102"),
            PV.StringFieldSchema("RegName_103"),
            PV.NumberFieldSchema("RegValue_103"),
            PV.StringFieldSchema("RegName_104"),
            PV.NumberFieldSchema("RegValue_104"),
            PV.StringFieldSchema("RegName_105"),
            PV.NumberFieldSchema("RegValue_105"),
            PV.StringFieldSchema("RegName_106"),
            PV.NumberFieldSchema("RegValue_106"),
            PV.StringFieldSchema("RegName_107"),
            PV.NumberFieldSchema("RegValue_107"),
            PV.StringFieldSchema("RegName_108"),
            PV.NumberFieldSchema("RegValue_108"),
            PV.StringFieldSchema("RegName_109"),
            PV.NumberFieldSchema("RegValue_109"),
            PV.StringFieldSchema("RegName_110"),
            PV.NumberFieldSchema("RegValue_110"),
            PV.StringFieldSchema("RegName_111"),
            PV.NumberFieldSchema("RegValue_111"),
            PV.StringFieldSchema("RegName_112"),
            PV.NumberFieldSchema("RegValue_112"),
            PV.StringFieldSchema("RegName_113"),
            PV.NumberFieldSchema("RegValue_113"),
            PV.StringFieldSchema("RegName_114"),
            PV.NumberFieldSchema("RegValue_114"),
            PV.StringFieldSchema("RegName_115"),
            PV.NumberFieldSchema("RegValue_115"),
            PV.StringFieldSchema("RegName_116"),
            PV.NumberFieldSchema("RegValue_116")
        )
     )
   );
};

